#ifndef _HTP_SDK_H_
#define _HTP_SDK_H_

#include <stdio.h>
#include <stdint.h>
#include <stddef.h>


enum InfoType {
    INFO_TYPE_HEARTBEAT = 1,
    INFO_TYPE_ENC_HEARTBEAT = 2,
    INFO_TYPE_CHEATINFO = 3,
};

#ifdef __cplusplus

class IInfoReceiver {
public:
    virtual ~IInfoReceiver() {}

    virtual void onReceive(int InfoType, const char *info) = 0;
};

#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @当游戏运行时进行初始化操作
 *
 * @param[in] game_key 请向易盾获取.
 * 
 * @注意：此Key与加固时使用的appkey不同
 */

void htp_init(const char *appid, const char *game_key, int server_type);

/**
 * @设置用户登录信息
 *
 * @param[in] role_id       角色id
 * @param[in] role_name     角色名称
 * @param[in] role_account  角色账号
 * @param[in] role_server   区服
 * @param[in] server_id     服务器ID
 * @param[in] game_json     对应设置游戏需要上传的信息，对应一个json字符串，具体说明请见文档
 *
 */
void htp_set_role_info(const char *role_id, const char *role_name, const char *role_account,
                       const char *role_server, int server_id, const char *game_json);

enum RequestCmdID {
    Cmd_GetEmulatorName = 1,
    Cmd_IsRootDevice = 2,
    Cmd_DeviceID = 3,
    Cmd_GetHTPVersion = 7,
    Cmd_GetCollectData = 8,
    Cmd_GetEncHTPVersion = 9,
    Cmd_GetToken = 10
};

/**
 * @通过此函数可以和反外挂进行交互
 *
 * @param[in] request       请求id
 * @param[in] data          额外数据，不需要为NULL
 * @param[out] buf          返回信息
 * @param[in] size          buf大小
 * @param[out] used_buf_len 使用buf大小
 *
 * @return 返回是否成功的标识 1为成功，0为失败
 *
 */
int htp_ioctl(int request, const char *data, /*[out]*/char *buf, size_t size, /*[out]*/
              size_t *used_buf_size);

int htp_getToken(/*[out]*/char *buf, size_t size, /*[out]*/
              size_t *used_buf_size);

/**
 * @通信校验相关（未使用）
 *
 * @param[in] alg_type      类型
 * @param[in] data          数据
 * @param[out] buf          返回数据
 * @param[in] size          buf大小
 * @param[out] used_buf_len 使用buf大小
 *
 */
int htp_get_data_sign(int alg_type, const char *data, /*[out]*/char *buf, size_t size,
                      size_t *used_buf_size);

/**
 * @单机游戏保护加解密
 *
 * @param[in] buf           需要保护的数据
 * @param[in] size          需要保护的数据的大小
 * @param[out] enc_buf      返回数据
 * @param[in] enc_buf_size  enc_buf大小
 * @param[out] used_buf_len 使用enc_buf的大小
 * 
 */
int
htp_enc_local(const char *buf, size_t size, /*[out]*/char *enc_buf, size_t enc_buf_size, /*[out]*/
              size_t *used_buf_size, int alg_type);

/**
 * @单机游戏保护加解密
 *
 * @param[in] buf           需要保护的数据
 * @param[in] size          需要保护的数据的大小
 * @param[out] dec_buf      返回数据
 * @param[in] dec_buf_size  dec_buf大小
 * @param[out] used_buf_len 使用dec_buf的大小
 * 
 */
int
htp_dec_local(const char *buf, size_t size, /*[out]*/char *dec_buf, size_t dec_buf_size, /*[out]*/
              size_t *used_buf_size, int alg_type);

/**
 * @自定义解密接口（加固后加密的数据进行解密）
 *
 * @param[in] buf           需要解密的数据
 * @param[in] buf_len       需要解密的数据大小
 * @param[out] dec_buf      返回数据
 * @param[in] dec_buf_len   dec_buf大小
 * @param[out] used_buf_len 使用dec_buf的大小
 *
 * @注意，返回数据内存大小和需要解密大小保持一致即可
 */
int htp_decrypt_buf(char *buf, size_t buf_len, /*[out]*/char *dec_buf, size_t dec_buf_len, /*[out]*/
                    size_t *used_buf_len);

/**
 * @加载白盒Aes密钥接口，一个密钥只需要加载一次，每次更新密钥之后，都需要重新加载
 * @param[in] bind          是否绑定设备，0：不绑定，1：绑定
 */
int htp_aes_load_table(int bind);

/**
 * @安全通信协议接口
 *
 * @param[in] alg_type      类型, 目前传0
 * @param[in] data          数据
 * @param[in] data_len      数据长度
 * @param[out] buf          返回数据
 * @param[in] size          buf大小
 * @param[out] used_buf_len 使用buf大小
 * @param[in] dec           1表示解密服务端的数据, 0表示加密数据到服务端
 * 
 * @注意，返回数据内存大小(size)需要比数据长度(data_len)至少大((data_len + 16)/3 * 4 + 100)
 * 即 size >= (data_len + 16)/3 * 4 + 100
 */
int htp_safe_comm(int alg_type, const char *data, size_t data_len, /*[out]*/char *buf,
                  size_t size, /*[out]*/size_t *used_buf_size, int dec);

/**
 * @模拟点击行为检测接口
 *
 * @param[in] gameplayId      玩法id
 * @param[in] sceneId         场景id
 */
void htp_register_touchevent(int gameplayId, int sceneId);

/**
 * @取消模拟点击行为检测接口
 *
 */
void htp_unregister_touchevent();

#ifdef __cplusplus
}
#endif

/**
 * @注册信息收集
 *
 * @param[in] receiver      接收器
 *
 * @接收器能够接受到发送的心跳数据
 *
 */
#ifdef __cplusplus

void htp_register_info_receiver(IInfoReceiver *receiver);

#endif
#endif /* _HTP_SDK_H_ */
